import { Client, Direction, Directions, Isochrone, RoutingJSAPIError, Isochrones, Matrix, } from "@routingjs/core";
import { decode } from "@googlemaps/polyline-codec";
const handleValhallaError = (error) => {
    var _a, _b, _c, _d;
    const props = {
        statusCode: (_a = error.response) === null || _a === void 0 ? void 0 : _a.status,
        status: (_b = error.response) === null || _b === void 0 ? void 0 : _b.statusText,
        errorCode: (_c = error.response) === null || _c === void 0 ? void 0 : _c.data.error_code,
        errorMessage: (_d = error.response) === null || _d === void 0 ? void 0 : _d.data.error,
    };
    throw new RoutingJSAPIError(error.message, props);
};
export class Valhalla {
    constructor(clientArgs) {
        const { apiKey, baseUrl, userAgent, headers, timeout, retryOverQueryLimit, maxRetries, axiosOpts, } = clientArgs || {};
        this.apiKey = apiKey; // TODO: add to requests
        const defaultURL = "https://valhalla1.openstreetmap.de";
        this.client = new Client(baseUrl || defaultURL, userAgent, timeout, retryOverQueryLimit, headers, maxRetries, axiosOpts);
    }
    async directions(locations, profile, directionsOpts = {}, dryRun = false) {
        dryRun = dryRun || false;
        const getParams = this.apiKey
            ? { access_token: this.apiKey }
            : undefined;
        const params = Valhalla.getDirectionParams(locations, profile, directionsOpts);
        return this.client
            .request({
            endpoint: "/route",
            postParams: params,
            getParams,
            dryRun,
        })
            .then((res) => {
            if (typeof res === "object") {
                return this.parseDirectionsResponse(res, "main");
            }
            else {
                return res; // return the request info string
            }
        })
            .catch(handleValhallaError);
    }
    static getDirectionParams(locations, profile, directionsOpts = {}) {
        const params = {
            locations: Valhalla._buildLocations(locations),
            costing: profile,
            narrative: directionsOpts.instructions || false,
        };
        if ((directionsOpts.costingOpts !== undefined &&
            Object.keys(directionsOpts.costingOpts).length) ||
            directionsOpts.preference !== undefined) {
            params.costing_options = {
                [profile]: Object.assign(Object.assign({}, directionsOpts.costingOpts), { shortest: directionsOpts.preference ? true : undefined }),
            };
        }
        if (directionsOpts.language ||
            directionsOpts.units ||
            directionsOpts.directionsType) {
            params.directions_options = {
                language: directionsOpts.language,
                units: directionsOpts.units,
                directions_type: directionsOpts.directionsType,
            };
        }
        if (directionsOpts.avoidLocations) {
            const avoidLocations = [];
            directionsOpts.avoidLocations.forEach((avoid_location) => {
                // TODO: convert to loop
                if (Array.isArray(avoid_location)) {
                    avoidLocations.push(avoid_location);
                }
                else if (avoid_location.type === "Feature") {
                    // GeoJSON Position object can have elevation coordinate
                    avoidLocations.push([
                        avoid_location.geometry.coordinates[0],
                        avoid_location.geometry.coordinates[1],
                    ]);
                }
                else {
                    // geometry obj only
                    avoidLocations.push([
                        avoid_location.coordinates[0],
                        avoid_location.coordinates[1],
                    ]);
                }
            });
            params.exclude_locations = avoidLocations;
        }
        if (directionsOpts.avoidPolygons) {
            const avoidPolygons = [];
            directionsOpts.avoidPolygons.forEach((avoid_polygon) => {
                // TODO: convert to loop
                if (Array.isArray(avoid_polygon)) {
                    avoidPolygons.push(avoid_polygon);
                }
                else if (avoid_polygon.type === "Feature") {
                    const outerRing = avoid_polygon.geometry.coordinates.map((ring) => {
                        return ring.map((pos) => {
                            return [pos[0], pos[1]]; // strip possible elevation
                        });
                    });
                    avoidPolygons.push(outerRing);
                }
                params.exclude_polygons = avoidPolygons;
            });
        }
        if (directionsOpts.dateTime) {
            params.date_time = directionsOpts.dateTime;
        }
        if (directionsOpts.id) {
            params.id = directionsOpts.id;
        }
        if (directionsOpts.alternatives) {
            params.alternates = directionsOpts.alternatives;
        }
        return params;
    }
    parseDirectionsResponse(response, type = "main") {
        var _a, _b, _c, _d;
        const geometry = [];
        let [duration, distance] = [0, 0];
        let factor = 1;
        if ((_a = response.trip) === null || _a === void 0 ? void 0 : _a.units) {
            factor = ["mi", "miles"].includes((_b = response.trip) === null || _b === void 0 ? void 0 : _b.units)
                ? 0.621371
                : 1;
        }
        (_d = (_c = response.trip) === null || _c === void 0 ? void 0 : _c.legs) === null || _d === void 0 ? void 0 : _d.forEach((leg) => {
            // TODO: convert to loop
            if (leg.shape) {
                geometry.push(...decode(leg.shape, 6).map(([lat, lon]) => [
                    lon,
                    lat,
                ]));
            }
            if (leg.summary.length) {
                distance += leg.summary.length * factor;
            }
            if (leg.summary.time) {
                duration += leg.summary.time;
            }
        });
        const feat = {
            type: "Feature",
            geometry: {
                type: "LineString",
                coordinates: geometry,
            },
            properties: { distance, duration },
        };
        if (type === "main") {
            const directions = response.alternates
                ? response.alternates.map((res) => {
                    return this.parseDirectionsResponse(res, "alternative");
                })
                : [];
            return new Directions([new Direction(feat, response), ...directions], response);
        }
        else {
            return new Direction(feat, response);
        }
    }
    async reachability(location, profile, intervals, isochronesOpts = {}, dryRun) {
        const getParams = this.apiKey
            ? { access_token: this.apiKey }
            : undefined;
        const params = Valhalla.getIsochroneParams(location, profile, intervals, isochronesOpts);
        return this.client
            .request({
            endpoint: "/isochrone",
            postParams: params,
            getParams,
            dryRun,
        })
            .then((res) => {
            if (typeof res === "object") {
                return Valhalla.parseIsochroneResponse(res, location, intervals, (isochronesOpts === null || isochronesOpts === void 0 ? void 0 : isochronesOpts.intervalType)
                    ? isochronesOpts.intervalType
                    : "time");
            }
            else {
                return res; // return the request info string
            }
        })
            .catch(handleValhallaError);
    }
    static getIsochroneParams(location, profile, intervals, isochroneOpts = {}) {
        const contours = [];
        const [key, divisor] = isochroneOpts.intervalType !== undefined &&
            isochroneOpts.intervalType === "distance"
            ? ["distance", 1000]
            : ["time", 60];
        intervals.forEach((interval, index) => {
            // TODO: convert to loop
            const contourObj = {
                [key]: interval / divisor,
            };
            if (isochroneOpts.colors !== undefined) {
                if (isochroneOpts.colors.length !== intervals.length) {
                    throw new Error("Colors array must be of same length as intervals array");
                }
                contourObj.color = isochroneOpts.colors[index];
            }
            contours.push(contourObj);
        });
        const params = {
            locations: Valhalla._buildLocations(location),
            costing: profile,
            contours,
            polygons: isochroneOpts.polygons,
        };
        if ((isochroneOpts.costingOpts !== undefined &&
            Object.keys(isochroneOpts.costingOpts).length) ||
            isochroneOpts.preference !== undefined)
            params.costing_options = {
                [profile]: Object.assign(Object.assign({}, isochroneOpts.costingOpts), { shortest: isochroneOpts.preference ? true : undefined }),
            };
        if (isochroneOpts.avoidLocations) {
            const avoidLocations = [];
            isochroneOpts.avoidLocations.forEach((avoid_location) => {
                // TODO: convert to loop
                if (Array.isArray(avoid_location)) {
                    avoidLocations.push(avoid_location);
                }
                else if (avoid_location.type === "Feature") {
                    // GeoJSON Position object can have elevation coordinate
                    avoidLocations.push([
                        avoid_location.geometry.coordinates[0],
                        avoid_location.geometry.coordinates[1],
                    ]);
                }
                else {
                    // geometry obj only
                    avoidLocations.push([
                        avoid_location.coordinates[0],
                        avoid_location.coordinates[1],
                    ]);
                }
            });
            params.exclude_locations = avoidLocations;
        }
        if (isochroneOpts.avoidPolygons) {
            const avoidPolygons = [];
            isochroneOpts.avoidPolygons.forEach((avoid_polygon) => {
                // TODO: convert to loop
                if (Array.isArray(avoid_polygon)) {
                    avoidPolygons.push(avoid_polygon);
                }
                else if (avoid_polygon.type === "Feature") {
                    const outerRing = avoid_polygon.geometry.coordinates.map((ring) => {
                        return ring.map((pos) => {
                            return [pos[0], pos[1]]; // strip possible elevation
                        });
                    });
                    avoidPolygons.push(outerRing);
                }
                params.exclude_polygons = avoidPolygons;
            });
        }
        if (isochroneOpts.dateTime) {
            params.date_time = isochroneOpts.dateTime;
        }
        if (isochroneOpts.id) {
            params.id = isochroneOpts.id;
        }
        if (isochroneOpts.showLocations) {
            params.show_locations = isochroneOpts.showLocations;
        }
        return params;
    }
    static parseIsochroneResponse(response, location, intervals, intervalType) {
        const isochrones = [];
        response.features.forEach((feature, index) => {
            if (feature.geometry.type !== "Point") {
                isochrones.push(new Isochrone(Array.isArray(location)
                    ? location
                    : [location.lat, location.lon], intervals[index], intervalType, feature));
            }
        });
        return new Isochrones(isochrones, response);
    }
    async matrix(locations, profile, matrixOpts = {}, dryRun) {
        const getParams = this.apiKey
            ? { access_token: this.apiKey }
            : undefined;
        const params = Valhalla.getMatrixParams(locations, profile, matrixOpts);
        return this.client
            .request({
            endpoint: "/sources_to_targets",
            postParams: params,
            getParams,
            dryRun,
        })
            .then((res) => {
            if (typeof res === "object") {
                return Valhalla.parseMatrixResponse(res, (matrixOpts === null || matrixOpts === void 0 ? void 0 : matrixOpts.units) ? matrixOpts.units : "km");
            }
            else {
                return res; // return the request info string
            }
        })
            .catch(handleValhallaError);
    }
    static getMatrixParams(locations, profile, matrixOpts = {}) {
        const matrixLocations = Valhalla._buildLocations(locations);
        let sourceCoords = matrixLocations;
        if (matrixOpts.sources) {
            sourceCoords = sourceCoords.filter((source, index) => {
                var _a;
                (_a = matrixOpts.sources) === null || _a === void 0 ? void 0 : _a.includes(index);
            });
        }
        let destCoords = matrixLocations;
        if (matrixOpts.destinations) {
            destCoords = destCoords.filter((source, index) => {
                var _a;
                (_a = matrixOpts.destinations) === null || _a === void 0 ? void 0 : _a.includes(index);
            });
        }
        const params = {
            costing: profile,
            sources: sourceCoords,
            targets: destCoords,
        };
        if ((matrixOpts.costingOpts !== undefined &&
            Object.keys(matrixOpts.costingOpts).length) ||
            matrixOpts.preference !== undefined) {
            params.costing_options = {
                [profile]: Object.assign(Object.assign({}, matrixOpts.costingOpts), { shortest: matrixOpts.preference ? true : undefined }),
            };
        }
        if (matrixOpts.avoidLocations) {
            const avoidLocations = [];
            matrixOpts.avoidLocations.forEach((avoid_location) => {
                // TODO: convert to loop
                if (Array.isArray(avoid_location)) {
                    avoidLocations.push(avoid_location);
                }
                else if (avoid_location.type === "Feature") {
                    // GeoJSON Position object can have elevation coordinate
                    avoidLocations.push([
                        avoid_location.geometry.coordinates[0],
                        avoid_location.geometry.coordinates[1],
                    ]);
                }
                else {
                    // geometry obj only
                    avoidLocations.push([
                        avoid_location.coordinates[0],
                        avoid_location.coordinates[1],
                    ]);
                }
            });
            params.exclude_locations = avoidLocations;
        }
        if (matrixOpts.avoidPolygons) {
            const avoidPolygons = [];
            matrixOpts.avoidPolygons.forEach((avoid_polygon) => {
                // TODO: convert to loop
                if (Array.isArray(avoid_polygon)) {
                    avoidPolygons.push(avoid_polygon);
                }
                else if (avoid_polygon.type === "Feature") {
                    const outerRing = avoid_polygon.geometry.coordinates.map((ring) => {
                        return ring.map((pos) => {
                            return [pos[0], pos[1]]; // strip possible elevation
                        });
                    });
                    avoidPolygons.push(outerRing);
                }
                params.exclude_polygons = avoidPolygons;
            });
        }
        if (matrixOpts.id) {
            params.id = matrixOpts.id;
        }
        return params;
    }
    static parseMatrixResponse(response, units) {
        const factor = units === "miles" || units === "mi" ? 0.621371 : 1;
        const durations = response.sources_to_targets.map((origin) => origin.map((dest) => dest.time));
        const distances = response.sources_to_targets.map((origin) => origin.map((dest) => {
            if (dest.distance !== undefined && dest.distance !== null) {
                return Math.round(dest.distance * 1000 * factor);
            }
            else {
                return null;
            }
        }));
        return new Matrix(durations, distances, response);
    }
    async mapMatch(locations, profile, traceRouteOpts = {}, dryRun) {
        const getParams = this.apiKey
            ? { access_token: this.apiKey }
            : undefined;
        const params = Valhalla.getTraceRouteParams(locations, profile, traceRouteOpts);
        return this.client
            .request({
            endpoint: "/trace_route",
            postParams: params,
            getParams,
            dryRun,
        })
            .then((res) => {
            if (typeof res === "object") {
                return this.parseDirectionsResponse(res, "main");
            }
            else {
                return res; // return the request info string
            }
        })
            .catch(handleValhallaError);
    }
    static getTraceRouteParams(locations, profile, traceRouteOpts = {}) {
        var _a, _b, _c;
        const params = {
            shape: this._buildLocations(locations),
            costing: profile,
        };
        if (traceRouteOpts.costingOpts !== undefined &&
            Object.keys(traceRouteOpts.costingOpts).length) {
            params.costing_options = {
                [profile]: Object.assign({}, traceRouteOpts.costingOpts),
            };
        }
        params.shape_match = traceRouteOpts.shapeMatch;
        params.begin_time = traceRouteOpts.beginTime;
        params.durations = traceRouteOpts.durations;
        params.use_timestamps = traceRouteOpts.useTimestamps;
        if (traceRouteOpts.traceOptions) {
            params.trace_options = {
                gps_accuracy: traceRouteOpts.traceOptions.gpsAccuracy,
                breakage_distance: traceRouteOpts.traceOptions.breakageDistance,
                search_radius: traceRouteOpts.traceOptions.searchRadius,
                interpolation_distance: traceRouteOpts.traceOptions.interpolationDistance,
            };
        }
        params.directions_options = {
            language: (_a = traceRouteOpts.directionsOptions) === null || _a === void 0 ? void 0 : _a.language,
            units: (_b = traceRouteOpts.directionsOptions) === null || _b === void 0 ? void 0 : _b.units,
            directions_type: (_c = traceRouteOpts.directionsOptions) === null || _c === void 0 ? void 0 : _c.directionsType,
        };
        params.id = traceRouteOpts.id;
        return params;
    }
    static _buildLocations(coordinates) {
        if (Array.isArray(coordinates)) {
            if (Array.isArray(coordinates[0])) {
                // [[lat, long], [lat, long], ...]
                const locations = [];
                coordinates.forEach((coordPair) => {
                    const locObj = { lon: coordPair[1], lat: coordPair[0] };
                    locations.push(locObj);
                });
                return locations;
            }
            else {
                if (typeof coordinates[0] == "number") {
                    // [lat, lng]
                    const location = [
                        {
                            lat: coordinates[0],
                            lon: coordinates[1],
                        },
                    ];
                    return location;
                }
                else {
                    // location objects
                    return coordinates;
                }
            }
        }
        else {
            // single location obj
            return [coordinates];
        }
    }
}
export * from "./parameters";
//# sourceMappingURL=index.js.map