import { Feature, FeatureCollection, Point, Polygon } from "geojson";
import { Client, Direction, Directions, RoutingJSAPIError, Isochrones, Matrix, BaseRouter, ClientConstructorArgs, ErrorProps } from "@routingjs/core";
import { MapboxAuthParams, ValhallaCostingOptsAuto, ValhallaCostingOptsBicycle, ValhallaCostingOptsMotorcycle, ValhallaCostingOptsPedestrian, ValhallaCostingOptsTruck, ValhallaCostingType, ValhallaDateTime, ValhallaDirectionsType, ValhallaIsochroneParams, ValhallaIsochroneResponse, ValhallaLocation, ValhallaMatrixParams, ValhallaMatrixResponse, ValhallaRequestUnit, ValhallaRouteParams, ValhallaRouteResponse, ValhallaShapeMatch, ValhallaTraceRouteParams } from "./parameters";
/**
 * `ValhallaErrorProps` returns additional information about the error thrown by the
 *  Valhalla routing engine. It sends back the status_code, status, error_code and error
 *  message where the error_code is specific to Valhalla.
 */
export interface ValhallaErrorProps extends ErrorProps {
    errorCode?: number;
}
export declare type ValhallaAPIError = RoutingJSAPIError<ValhallaErrorProps>;
interface ValhallaBaseOpts {
    /** A request ID that will be returned in the response */
    id?: string;
    /**
     * Convenience argument to set the cost metric, one of ['shortest', 'fastest']. Note,
            that shortest is not guaranteed to be absolute shortest for motor vehicle profiles. It's called ``preference``
            to be in line with the already existing parameter in the ORS adapter.
     */
    preference?: "shortest" | "fastest";
    /**
     * Profiles can have several options that can be adjusted to develop the route path,
            as well as for estimating time along the path. Only specify the actual options dict, the profile
            will be filled automatically. For more information, visit:
            https://github.com/valhalla/valhalla/blob/master/docs/api/turn-by-turn/api-reference.md#costing-options
     */
    costingOpts?: ValhallaCostingOptsAuto | ValhallaCostingOptsTruck | ValhallaCostingOptsBicycle | ValhallaCostingOptsMotorcycle | ValhallaCostingOptsPedestrian;
    /**
     * A set of locations to exclude or avoid within a route.
     */
    avoidLocations?: ([number, number] | Point | Feature<Point>)[];
    /**
     * Roads intersecting these polygons
            will be avoided during path finding. If you only need to avoid a few specific roads, it's much more
            efficient to use avoid_locations. Valhalla will close open rings (i.e. copy the first coordingate to the
            last position).
     */
    avoidPolygons?: ([number, number][][] | Polygon | Feature<Polygon>)[];
    /**
     * This is the local date and time at the location. Field `type`: 0: Current departure time,
       1: Specified departure time. Field `value`: the date and time is specified
       in ISO 8601 format (YYYY-MM-DDThh:mm), local time.

        @example
        ```js
        date_time = {type: 0, value: 2021-03-03T08:06:23}
        ```
     */
    dateTime?: ValhallaDateTime;
}
interface ValhallaAdditionalTraceOpts {
    /** Search radius in meters associated with supplied trace points. */
    searchRadius?: number;
    /** GPS accuracy in meters associated with supplied trace points. */
    gpsAccuracy?: number;
    /** Breaking distance in meters between trace points.  */
    breakageDistance?: number;
    /** Interpolation distance in meters beyond which trace points are merged together. */
    interpolationDistance?: number;
    /**
     * When present and true, the successful trace_route response will include a key
     * linear_references. Its value is an array of base64-encoded [OpenLR](https://www.openlr-association.com/fileadmin/user_upload/openlr-whitepaper_v1.5.pdf) location
     * references, one for each graph edge of the road network matched by the input trace.
     *
     */
    linearReferences?: boolean;
}
export interface ValhallaDirectionOpts extends ValhallaBaseOpts {
    /**
     * Whether to return turn-by-turn instructions. Named for compatibility with other
     *  providers. Valhalla's parameter here is 'narrative'.
     */
    instructions?: boolean;
    /** A number denoting how many alternate routes should be provided.
     * There may be no alternates or less alternates than the user specifies.
     *  Alternates are not yet supported on multipoint routes (that is, routes with
     * more than 2 locations). They are also not supported on time dependent routes.
     */
    alternatives?: number;
    /**
     * Distance units for output. Allowable unit types are miles (or mi) and kilometers (or km).
     * If no unit type is specified, the units default to kilometers.
     */
    units?: ValhallaRequestUnit;
    /**
     * The language of the narration instructions based on the IETF BCP 47 language tag string.
     * If no language is specified or the specified language is unsupported, United States-based
     * English (en-US) is used.
     *
     * See here for a list of supported languages: {@link https://valhalla.readthedocs.io/en/latest/api/turn-by-turn/api-reference/#supported-language-tags}
     */
    language?: string;
    /**
     * 'none': no instructions are returned. 'maneuvers': only maneuvers are returned.
     * 'instructions': maneuvers with instructions are returned. Default 'instructions'
     */
    directionsType?: ValhallaDirectionsType;
}
export interface ValhallaIsochroneOpts extends ValhallaBaseOpts {
    /**
     * Set 'time' for isochrones or 'distance' for equidistants.
     * Default 'time'.
     */
    intervalType?: "time" | "distance";
    /**
     * The color for the output of the contour. Specify it as a Hex value, but without the #, such as
     * "color":"ff0000" for red. If no color is specified, the isochrone service will assign a default color to the output.
     */
    colors?: string[];
    /**
     * Controls whether polygons or linestrings are returned in GeoJSON geometry. Default False.
     */
    polygons?: boolean;
    /**
     * Can be used to remove smaller contours. In range [0, 1]. A value of 1 will only return the largest contour
     * for a given time value. A value of 0.5 drops any contours that are less than half the area of the largest
     * contour in the set of contours for that same time value. Default 1.
     */
    denoise?: number;
    /**
     * A floating point value in meters used as the tolerance for Douglas-Peucker generalization.
     * Note: Generalization of contours can lead to self-intersections, as well as intersections of adjacent contours.
     */
    generalize?: number;
    /**
     * A boolean indicating whether the input locations should be returned as MultiPoint features: one feature for the exact input coordinates and one feature
     * for the coordinates of the network node it snapped to. Default false.
     */
    showLocations?: boolean;
}
export interface ValhallaMatrixOpts extends ValhallaBaseOpts {
    sources?: number[];
    destinations?: number[];
    units?: ValhallaRequestUnit;
}
export interface ValhallaTraceRouteOpts extends Omit<ValhallaBaseOpts, "avoidLocations" | "avoidPolygons"> {
    /**
     * shape_match is an optional string input parameter. It allows some control
     * of the matching algorithm based on the type of input.
     *
     * @remarks
     * `edge_walk`:    Indicates an edge walking algorithm can be used. This algorithm
     *                 requires nearly exact shape matching, so it should only be used
     *                 when the shape is from a prior Valhalla route.
     *
     * `map_snap`: 	   Indicates that a map-matching algorithm should be used because
     *                 the input shape might not closely match Valhalla edges. This
     *                 algorithm is more expensive.
     *
     * `walk_or_snap`: Also the default option. This will try edge walking and if this
     *                 does not succeed, it will fall back and use map matching.
     *
     * @defaultValue
     * `walk_or_snap`
     */
    shapeMatch?: ValhallaShapeMatch;
    /**
     * Begin timestamp for the trace.
     *
     * @remarks
     * This is used along with the durations so that timestamps can be specified for a
     * trace that is specified using an encoded polyline.
     */
    beginTime?: string;
    /**
     * List of durations (seconds) between each successive pair of input trace points.
     *
     * @remarks
     * This allows trace points to be supplied as an encoded polyline and timestamps to be
     * created by using this list of "delta" times along with the begin_time of the trace.
     */
    durations?: number[];
    /**
     * A boolean value indicating whether the input timestamps or durations should be used
     * when computing elapsed time at each edge along the matched path.
     *
     * @remarks
     * If true, timestamps are used. If false (default), internal costing is applied to
     * compute elapsed times.
     *
     * @defaultValue
     * `false`
     */
    useTimestamps?: boolean;
    /** Additional Options */
    traceOptions?: ValhallaAdditionalTraceOpts;
    directionsOptions?: ValhallaDirectionOpts;
}
export declare type ValhallaDirections = Directions<ValhallaRouteResponse, ValhallaRouteResponse>;
export declare type ValhallaIsochrones = Isochrones<ValhallaIsochroneResponse, Feature>;
export declare type ValhallaMatrix = Matrix<ValhallaMatrixResponse>;
export declare type ValhallaTraceRoute = Directions<ValhallaRouteResponse, ValhallaRouteResponse>;
export declare type ValhallaClient = Client<ValhallaRouteResponse | ValhallaMatrixResponse | FeatureCollection, MapboxAuthParams, ValhallaIsochroneParams | ValhallaRouteParams | ValhallaMatrixParams | ValhallaTraceRouteParams>;
export declare class Valhalla implements BaseRouter<ValhallaLocation> {
    client: ValhallaClient;
    apiKey?: string;
    constructor(clientArgs?: ClientConstructorArgs);
    /**
     * Makes a request to Valhalla's `/route` endpoint.
     *
     * @param locations - The coordinates tuple the route should be calculated from in order of visit. Format: [lat, lon]
     * @param profile - Specifies the mode of transport
     * @param directionsOpts - Additional parameters, such as costing options.
     * @param dryRun - if true, will not make the request and instead return an info string containing the URL and request parameters; for debugging
     * @see {@link ValhallaCostingType} for available profiles
     */
    directions(locations: ([number, number] | ValhallaLocation)[], profile: ValhallaCostingType, directionsOpts?: ValhallaDirectionOpts, dryRun?: false): Promise<ValhallaDirections>;
    directions(locations: ([number, number] | ValhallaLocation)[], profile: ValhallaCostingType, directionsOpts: ValhallaDirectionOpts, dryRun: true): Promise<string>;
    static getDirectionParams(locations: ([number, number] | ValhallaLocation)[], profile: ValhallaCostingType, directionsOpts?: ValhallaDirectionOpts): ValhallaRouteParams;
    parseDirectionsResponse(response: ValhallaRouteResponse, type: "main"): ValhallaDirections;
    parseDirectionsResponse(response: ValhallaRouteResponse, type: "alternative"): Direction<ValhallaRouteResponse>;
    /**
     * Makes a request to Valhalla's `/isochrone` endpoint.
     *
     * @param location - The coordinates tuple that represents the starting location. Format: [lat, lon]
     * @param profile - Specifies the mode of transport
     * @param isochronesOpts - Additional parameters, such as costing options.
     * @param dryRun - if true, will not make the request and instead return an info string containing the URL and request parameters; for debugging
     * @see {@link ValhallaCostingType} for available profiles
     */
    reachability(location: [number, number] | ValhallaLocation, profile: ValhallaCostingType, intervals: number[], isochronesOpts?: ValhallaIsochroneOpts, dryRun?: false): Promise<ValhallaIsochrones>;
    reachability(location: [number, number] | ValhallaLocation, profile: ValhallaCostingType, intervals: number[], isochronesOpts: ValhallaIsochroneOpts, dryRun: true): Promise<string>;
    static getIsochroneParams(location: [number, number] | ValhallaLocation, profile: ValhallaCostingType, intervals: number[], isochroneOpts?: ValhallaIsochroneOpts): ValhallaIsochroneParams;
    static parseIsochroneResponse(response: ValhallaIsochroneResponse, location: [number, number] | ValhallaLocation, intervals: number[], intervalType: "time" | "distance"): ValhallaIsochrones;
    /**
     * Makes a request to Valhalla's `/matrix` endpoint.
     *
     * @param locations - Format: [lat, lon]
     * @param profile - Specifies the mode of transport
     * @param matrixOpts - Additional parameters, such as costing options.
     * @param dryRun - if true, will not make the request and instead return an info string containing the URL and request parameters; for debugging
     * @see {@link ValhallaCostingType} for available profiles
     */
    matrix(locations: ([number, number] | ValhallaLocation)[], profile: ValhallaCostingType, matrixOpts?: ValhallaMatrixOpts, dryRun?: false): Promise<ValhallaMatrix>;
    matrix(locations: ([number, number] | ValhallaLocation)[], profile: ValhallaCostingType, matrixOpts: ValhallaMatrixOpts, dryRun: true): Promise<string>;
    static getMatrixParams(locations: ([number, number] | ValhallaLocation)[], profile: ValhallaCostingType, matrixOpts?: ValhallaMatrixOpts): ValhallaMatrixParams;
    static parseMatrixResponse(response: ValhallaMatrixResponse, units: ValhallaRequestUnit): ValhallaMatrix;
    /**
     * Makes a request to Valhalla's `/trace_route` endpoint.
     *
     * @param locations - Format: [lat, lon]
     * @param profile - Specifies the mode of transport
     * @param mapMatchOpts - Additional parameters, such as costing options.
     * @param dryRun - if true, will not make the request and instead return an info string containing the URL and request parameters; for debugging
     * @see {@link ValhallaCostingType} for available profiles
     */
    mapMatch(locations: ([number, number] | ValhallaLocation)[], profile: ValhallaCostingType, mapMatchOpts?: ValhallaTraceRouteOpts, dryRun?: false): Promise<ValhallaDirections>;
    mapMatch(locations: ([number, number] | ValhallaLocation)[], profile: ValhallaCostingType, mapMatchOpts: ValhallaTraceRouteOpts, dryRun: true): Promise<string>;
    static getTraceRouteParams(locations: ([number, number] | ValhallaLocation)[], profile: ValhallaCostingType, traceRouteOpts?: ValhallaTraceRouteOpts): ValhallaTraceRouteParams;
    static _buildLocations(coordinates: ([number, number] | ValhallaLocation)[] | [number, number] | ValhallaLocation): ValhallaLocation[];
    static _buildLocations(coordinates: ([number, number] | ValhallaLocation)[] | [number, number] | ValhallaLocation): [ValhallaLocation];
}
export * from "./parameters";
//# sourceMappingURL=index.d.ts.map